[PATCH] D86541: Preserve vcc_lo when shrinking V_CNDMASK

Piotr Sobczak via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Wed Aug 26 03:09:33 PDT 2020


piotr updated this revision to Diff 287895.
piotr added a comment.

As suggested, moved the code to buildShrunkInst where the instruction is created.


Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D86541/new/

https://reviews.llvm.org/D86541

Files:
  llvm/lib/Target/AMDGPU/SIInstrInfo.cpp
  llvm/test/CodeGen/AMDGPU/shrink-instructions-implicit-vcclo.mir


Index: llvm/test/CodeGen/AMDGPU/shrink-instructions-implicit-vcclo.mir
===================================================================
--- /dev/null
+++ llvm/test/CodeGen/AMDGPU/shrink-instructions-implicit-vcclo.mir
@@ -0,0 +1,22 @@
+# RUN: llc -march=amdgcn -mcpu=gfx1010 -run-pass=si-shrink-instructions --verify-machineinstrs %s -o - | FileCheck %s
+
+# Make sure the implicit vcc_lo of V_CNDMASK is preserved and not promoted to vcc.
+---
+
+name:            shrink_cndmask_implicit_vcc_lo
+tracksRegLiveness: true
+body:             |
+  bb.0:
+    liveins: $vgpr0, $vgpr1
+    ; CHECK-LABEL: name: shrink_cndmask_implicit_vcc_lo
+    ; CHECK: [[COPY:%[0-9]+]]:vgpr_32 = COPY $vgpr0
+    ; CHECK: [[COPY1:%[0-9]+]]:vgpr_32 = COPY $vgpr0
+    ; CHECK: V_CMP_LT_I32_e32 0, [[COPY]], implicit-def $vcc_lo, implicit $exec
+    ; CHECK: V_CNDMASK_B32_e32 0, [[COPY1]], implicit $vcc_lo, implicit $exec
+    %0:vgpr_32 = COPY $vgpr0
+    %1:vgpr_32 = COPY $vgpr0
+    V_CMP_LT_I32_e32 0, %0:vgpr_32, implicit-def $vcc_lo, implicit $exec, implicit-def $vcc
+    %2:vgpr_32 = V_CNDMASK_B32_e64 0, 0, 0, %1:vgpr_32, $vcc_lo, implicit $exec
+    S_NOP 0
+
+...
Index: llvm/lib/Target/AMDGPU/SIInstrInfo.cpp
===================================================================
--- llvm/lib/Target/AMDGPU/SIInstrInfo.cpp
+++ llvm/lib/Target/AMDGPU/SIInstrInfo.cpp
@@ -3467,8 +3467,18 @@
       Inst32.add(*Src2);
     } else {
       // In the case of V_CNDMASK_B32_e32, the explicit operand src2 is
-      // replaced with an implicit read of vcc. This was already added
-      // during the initial BuildMI, so find it to preserve the flags.
+      // replaced with an implicit read of vcc or vcc_lo. The implicit read
+      // of vcc was already added during the initial BuildMI, but we
+      // 1) may need to change vcc to vcc_lo to preserve the original register
+      // 2) have to preserve the original flags.
+
+      if (Src2->getReg() == AMDGPU::VCC_LO) {
+        for (auto &Use : Inst32->implicit_operands()) {
+          if (Use.isUse() && Use.getReg() == AMDGPU::VCC)
+            Use.setReg(AMDGPU::VCC_LO);
+        }
+      }
+
       copyFlagsToImplicitVCC(*Inst32, *Src2);
     }
   }


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D86541.287895.patch
Type: text/x-patch
Size: 2205 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20200826/2059ed5c/attachment.bin>


More information about the llvm-commits mailing list